using System;
using System.Web.Mvc;
using ChartDirector;

namespace NetMvcCharts.Controllers
{
    public class RealtimemultichartController : Controller
    {
        //
        // Draw a single chart
        //
        private XYChart drawXYChart(RazorChartViewer viewer, DateTime[] timeStamps, double[] dataSeries,
            string name, int color, Axis xAxisScale, bool xAxisVisible)
        {
            // Only the last chart has an x-axis
            int xAxisHeight = 25;
            if (!xAxisVisible) {
                xAxisHeight = 0;
            }

            // Create an XYChart object of size 640 x 120 pixels (excluding x-axis height)
            XYChart c = new XYChart(640, 120 + xAxisHeight);

            // Set the plotarea at (55, 10) with width 85 pixels less than chart width, and height 20
            // pixels less than chart height. Use a vertical gradient from light blue (f0f6ff) to sky
            // blue (a0c0ff) as background. Set border to transparent and grid lines to white (ffffff).
            c.setPlotArea(55, 10, c.getWidth() - 85, c.getHeight() - 20 - xAxisHeight,
                c.linearGradientColor(0, 10, 0, c.getHeight() - 20 - xAxisHeight, 0xf0f6ff, 0xa0c0ff),
                -1, Chart.Transparent, 0xffffff, 0xffffff);

            // As the data can lie outside the plotarea in a zoomed chart, we need enable clipping.
            c.setClipping();

            // Add a legend box at (55, 5) using horizontal layout. Use 8pts Arial Bold as font. Set the
            // background and border color to Transparent and use line style legend key.
            LegendBox b = c.addLegend(55, 5, false, "Arial Bold", 10);
            b.setBackground(Chart.Transparent);
            b.setLineStyleKey();

            // Set the x and y axis stems to transparent and the label font to 10pt Arial
            c.xAxis().setColors(Chart.Transparent);
            c.yAxis().setColors(Chart.Transparent);
            c.xAxis().setLabelStyle("Arial", 10);
            c.yAxis().setLabelStyle("Arial", 10);

            // Add axis title using 10pts Arial Bold Italic font
            c.yAxis().setTitle(name, "Arial Bold", 10);

            //================================================================================
            // Add data to chart
            //================================================================================

            // Add a line layer with the given data, with a line width of 2 pixels.
            LineLayer layer = c.addLineLayer();
            layer.setLineWidth(2);
            layer.setXData(timeStamps);
            layer.addDataSet(dataSeries, color, name);

            //================================================================================
            // Configure axis scale and labelling
            //================================================================================

            // For the automatic axis labels, set the minimum spacing to 30 pixels for the y axis.
            c.yAxis().setTickDensity(30);

            if (xAxisScale == null) {
                // If xAxisScale is given, then use it to synchronize with other charts.
                c.xAxis().copyAxis(xAxisScale);
            } else {
                // Set the x-axis label format
                c.xAxis().setLabelFormat("{value|hh:nn:ss}");
            }

            // Hide the x-axis if it is not visible.
            if (!xAxisVisible) {
                c.xAxis().setColors(Chart.Transparent, Chart.Transparent);
            }

            //================================================================================
            // Output the chart
            //================================================================================

            return c;
        }

        //
        // Draw the MultiChart
        //
        private void drawChart(RazorChartViewer viewer)
        {
            //
            // Data to draw the chart. In this demo, the data buffer will be filled by a random data
            // generator. In real life, the data is probably stored in a buffer (eg. a database table, a
            // text file, or some global memory) and updated by other means.
            //

            // We use a data buffer to emulate the last 240 samples.
            int sampleSize = 240;
            double[] dataSeries1 = new double[sampleSize];
            double[] dataSeries2 = new double[sampleSize];
            double[] dataSeries3 = new double[sampleSize];
            DateTime[] timeStamps = new DateTime[sampleSize];

            // Our pseudo random number generator
            DateTime firstDate = DateTime.Now.AddSeconds(-timeStamps.Length);
            for(int i = 0; i < timeStamps.Length; ++i) {
                timeStamps[i] = firstDate.AddSeconds(i);
                double p = timeStamps[i].Ticks / 10000000;
                dataSeries1[i] = Math.Cos(p * 2.1) * 10 + 1 / (Math.Cos(p) * Math.Cos(p) + 0.01) + 20;
                dataSeries2[i] = 100 * Math.Sin(p / 27.7) * Math.Sin(p / 10.1) + 150;
                dataSeries3[i] = 100 * Math.Cos(p / 6.7) * Math.Cos(p / 11.9) + 150;
            }

            // The MultiChart, initially set to a height 10 pixels as the top margin.
            MultiChart m = new MultiChart(640, 10);

            // This first chart is responsible for setting up the x-axis scale.
            XYChart xyc = drawXYChart(viewer, timeStamps, dataSeries1, "Alpha", 0xff0000, null, false);
            Axis xAxisScale = xyc.xAxis();

            // Add the XYChart to the MultiChart and increase the MultiChart height
            m.addChart(0, m.getHeight(), xyc);
            m.setSize(m.getWidth(), m.getHeight() + xyc.getHeight());

            // All other charts synchronize their x-axes with that of the first chart.
            xyc = drawXYChart(viewer, timeStamps, dataSeries2, "Beta", 0x008800, xAxisScale, false);

            // Add the XYChart to the MultiChart and increase the MultiChart height
            m.addChart(0, m.getHeight(), xyc);
            m.setSize(m.getWidth(), m.getHeight() + xyc.getHeight());

            // The last chart displays the x-axis.
            xyc = drawXYChart(viewer, timeStamps, dataSeries3, "Gamma", 0x0000ff, xAxisScale, true);

            // Add the XYChart to the MultiChart and increase the MultiChart height
            m.addChart(0, m.getHeight(), xyc);
            m.setSize(m.getWidth(), m.getHeight() + xyc.getHeight());

            // Set the combined plot area to be the bounding box of the plot areas of the 3 charts
            m.setMainChart(m);

            // Output the chart
            viewer.Image = m.makeWebImage(Chart.SVG);

            // Output Javascript chart model to the browser to suppport tracking cursor
            viewer.ChartModel = m.getJsChartModel();
        }

        public ActionResult Index()
        {
            RazorChartViewer viewer = ViewBag.Viewer = new RazorChartViewer(HttpContext, "chart1");

            //
            // This script handles both the full page request, as well as the subsequent partial updates
            // (AJAX chart updates). We need to determine the type of request first before we processing
            // it.
            //
            if (RazorChartViewer.IsPartialUpdateRequest(Request)) {
                // Is a partial update request.
                drawChart(viewer);
                return Content(viewer.PartialUpdateChart());
            }

            //
            // If the code reaches here, it is a full page request.
            //

            drawChart(viewer);

            return View();
        }
    }
}

