﻿Imports ChartDirector
Imports System.Collections.Generic
Imports System.Threading.Tasks
Imports System.Threading

Public Class FrmMegaRealTimeZoomScroll

    ' In this example, we plot 3 real-time series, each with 10,000,000 points maximum.
    ' For demonstration purpose, we will pre-load half of the buffer with random data.
    ' The remaining half wlll be filled with real-time random data at a rate of 1000 
    ' points per series per second. 
    Private Const bufferSize As Integer = 10000000

    ' Data arrays
    Private timeStamps(bufferSize - 1) As Double
    Private dataSeriesA(bufferSize - 1) As Double
    Private dataSeriesB(bufferSize - 1) As Double
    Private dataSeriesC(bufferSize - 1) As Double

    ' The index position that New data will be added.
    Private currentIndex As Integer

    ' Data Accelerator for handle large data sets
    Private fastData As DataAccelerator

    ' Flag to indicate the chart has been drawn so the zoom/scroll And track cursor
    ' event handlers can run.
    Private hasFinishedInitialization As Boolean = False

    ' The full x-axis range Is at least 60 seconds (60000ms), And the maximum zooming
    ' Is 10ms visible x-axis range.
    Private minXRange As Integer = 60000
    Private zoomInLimit As Integer = 10

    ' Real-time random number generator
    Private realTimeData As RanSeries = New RanSeries(999)
    Private dataSourceEmulator As Stopwatch = New Stopwatch()
    Private lastTimeStamp As Double = 0

    '
    ' For load event handler
    '
    Private Sub FrmMegaRealTimeZoomScroll_Load(sender As Object, e As EventArgs) _
        Handles MyBase.Load

        ' Pre-load half of the buffer with random data
        loadData()

        ' Display initial Message
        Dim c As PieChart = New PieChart(800, 400, &HD0E0FF)
        c.addTitle(Chart.Center, "<*block,halign=left,maxwidth=500*>" &
            "<*font=Arial Bold,size=18,underline=2*>Mega Real-Time Chart<*/font*><*br*><*br*>" &
            "This example demonstrates a real time chart with huge amount of data. It contains " &
            "3 lines pre-loaded with 5 million points each, then 1000 points are added to each " &
            "line per second. We limit it to 3 x 10 million points so that it needs less than " &
            "1G of RAM (320M to store the data, 600M to plot the data and handle the GUI)." &
            "<*br*><*br*><*br*>Press the Plot Chart button to plot the chart.")
        winChartViewer1.Image = c.makeImage()

    End Sub

    '
    ' Pre-load half of the buffer with random data
    '
    Private Sub loadData()

        Dim initialSize As Integer = bufferSize / 2

        ' To speed up random number generation, we use 3 threads to generate the random data
        ' for the 3 data series. The current thread Is used for generating the timestamps.
        Dim tasks As New List(Of Task)

        tasks.Add(Task.Factory.StartNew(
            Sub()
                Dim rA As RanSeries = New RanSeries(109)
                rA.fillSeries2(dataSeriesA, 0, initialSize, 2500, -1, 1)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                Dim rB As New RanSeries(110)
                rB.fillSeries2(dataSeriesB, 0, initialSize, 2500, -1, 1)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                Dim rC As New RanSeries(111)
                rC.fillSeries2(dataSeriesC, 0, initialSize, 2500, -1, 1)
            End Sub
        ))

        Dim rT As New RanSeries(0)
        rT.fillDateSeries(timeStamps, 0, initialSize, 0, 1)
        currentIndex = initialSize

        Task.WaitAll(tasks.ToArray())

    End Sub

    '
    ' User clicks on the Plot Chart pushbutton
    ' 
    Private Sub plotChartPB_Click(sender As Object, e As EventArgs) Handles plotChartPB.Click

        If hasFinishedInitialization Then
            Exit Sub
        End If

        ' This example uses a DataAccelerator to speed up the large amount of data. As we
        ' pre-load half of the buffer with random data, we need to process them first. 
        ' To speed up, we create two threads to process two of the data series, And use the
        ' current thread to process the third series.
        Dim tasks As New List(Of Task)

        fastData = New DataAccelerator(timeStamps, currentIndex)
        tasks.Add(Task.Factory.StartNew(
            Sub()
                fastData.setDataSeries("mA", dataSeriesA)
            End Sub
        ))
        tasks.Add(Task.Factory.StartNew(
            Sub()
                fastData.setDataSeries("mB", dataSeriesB)
            End Sub
        ))
        fastData.setDataSeries("mC", dataSeriesC)
        Task.WaitAll(tasks.ToArray())

        ' Initialize the WinChartViewer
        initChartViewer(winChartViewer1)
        hasFinishedInitialization = True

        ' Trigger drawing of the initial chart
        If currentIndex > 0 Then
            updateAxisScale(winChartViewer1)
        Else
            winChartViewer1.updateViewPort(True, True)
        End If

        ' Start the realtime data generation timer And chart update timer
        dataSourceEmulator.Start()
        chartUpdateTimer.Start()

    End Sub

    '
    ' Initialize the WinChartViewer
    '
    Private Sub initChartViewer(viewer As WinChartViewer)

        If currentIndex > 0 Then
            ' Set the full x range to be the data range, Or at least minXRange.
            Dim duration As Double = Math.Max(timeStamps(currentIndex - 1) - timeStamps(0), minXRange)
            viewer.setFullRange("x", timeStamps(0), timeStamps(0) + duration)

            ' Initialize the view port to show the latest 20% of the x range, Or at least minXRange.
            viewer.ViewPortWidth = Math.Max(0.2, minXRange / duration)
            viewer.ViewPortLeft = 1 - viewer.ViewPortWidth

            ' Set the maximum zoom to 10 x-units
            viewer.ZoomInWidthLimit = Math.Min(1.0, zoomInLimit / duration)
        Else
            viewer.ViewPortLeft = 0
            viewer.ViewPortWidth = 1
        End If

        ' Enable mouse wheel zooming by setting the zoom ratio to 1.1 per wheel event
        viewer.MouseWheelZoomRatio = 1.1

        ' Initially set the mouse usage to Drag to Scroll mode
        pointerPB.Checked = True

    End Sub

    '
    ' Pointer (Drag to Scroll) button event handler
    '
    Private Sub pointerPB_CheckedChanged(sender As Object, e As EventArgs) _
        Handles pointerPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ScrollOnDrag
        End If

    End Sub

    '
    ' Zoom In button event handler
    '
    Private Sub zoomInPB_CheckedChanged(sender As Object, e As EventArgs) _
        Handles zoomInPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ZoomIn
        End If

    End Sub

    '
    ' Zoom Out button event handler
    '
    Private Sub zoomOutPB_CheckedChanged(sender As Object, e As EventArgs) _
        Handles zoomOutPB.CheckedChanged

        If sender.Checked Then
            winChartViewer1.MouseUsage = WinChartMouseUsage.ZoomOut
        End If

    End Sub

    '
    ' The ViewPortChanged event handler. This event occurs if the user scrolls or zooms in
    ' or out the chart by dragging or clicking on the chart. It can also be triggered by
    ' calling WinChartViewer.updateViewPort.
    '
    Private Sub winChartViewer1_ViewPortChanged(sender As Object, e As WinViewPortEventArgs) _
        Handles winChartViewer1.ViewPortChanged

        If Not hasFinishedInitialization Then
            Exit Sub
        End If

        updateControls(winChartViewer1)

        If e.NeedUpdateChart Then
            drawChart(winChartViewer1)
        End If

    End Sub


    '
    ' Update controls when the view port changed
    '
    Private Sub updateControls(viewer As WinChartViewer)

        ' In this demo, we need to update the scroll bar to reflect the view port position and
        ' width of the view port.
        hScrollBar1.Enabled = viewer.ViewPortWidth < 1
        hScrollBar1.LargeChange = Math.Ceiling(viewer.ViewPortWidth *
            (hScrollBar1.Maximum - hScrollBar1.Minimum))
        hScrollBar1.SmallChange = Math.Ceiling(hScrollBar1.LargeChange * 0.1)
        hScrollBar1.Value = Math.Round(viewer.ViewPortLeft *
            (hScrollBar1.Maximum - hScrollBar1.Minimum)) + hScrollBar1.Minimum

    End Sub

    '
    ' The scroll bar event handler
    '
    Private Sub hScrollBar1_ValueChanged(sender As Object, e As EventArgs) _
        Handles hScrollBar1.ValueChanged

        ' When the view port is changed (user drags on the chart to scroll), the scroll bar will get
        ' updated. When the scroll bar changes (eg. user drags on the scroll bar), the view port will
        ' get updated. This creates an infinite loop. To avoid this, the scroll bar can update the 
        ' view port only if the view port is not updating the scroll bar.
        If hasFinishedInitialization And Not winChartViewer1.IsInViewPortChangedEvent Then
            ' Set the view port based on the scroll bar
            winChartViewer1.ViewPortLeft = (hScrollBar1.Value - hScrollBar1.Minimum) /
                (hScrollBar1.Maximum - hScrollBar1.Minimum)

            ' Trigger a view port changed event to update the chart
            winChartViewer1.updateViewPort(True, False)
        End If

    End Sub

    '
    ' Draw the chart And display it in the given viewer
    '
    Private Sub drawChart(viewer As WinChartViewer)

        ' Get the start date and end date that are visible on the chart.
        Dim viewPortStartDate As Double = viewer.getValueAtViewPort("x", viewer.ViewPortLeft)
        Dim viewPortEndDate As Double = viewer.getValueAtViewPort("x", viewer.ViewPortRight)
        fastData.setSubsetRange(viewPortStartDate, viewPortEndDate)

        '
        ' At this stage, we have extracted the visible data. We can use those data to plot the chart.
        '

        '================================================================================
        ' Configure overall chart appearance.
        '================================================================================
        ' Create an XYChart object of size 800 x 400 pixels
        Dim c As XYChart = New XYChart(800, 400)

        ' Set the plotarea at (0, 0) with width 1 pixel less than chart width, And height 20 pixels
        ' less than chart height. Use a vertical gradient from light blue (f0f6ff) to sky blue (a0c0ff)
        ' as background. Set border to transparent And grid lines to white (ffffff).
        c.setPlotArea(0, 0, c.getWidth() - 1, c.getHeight() - 20, c.linearGradientColor(0, 0, 0,
            c.getHeight() - 20, &HF0F6FF, &HA0C0FF), -1, Chart.Transparent, &HFFFFFF, &HFFFFFF)

        ' As the data can lie outside the plotarea in a zoomed chart, we need enable clipping.
        c.setClipping()

        ' Add a legend box at the right side using horizontal layout. Use 10pt Arial Bold as font. Set
        ' the background And border color to Transparent And use line style legend key.
        Dim b As LegendBox = c.addLegend(c.getWidth() - 1, 10, False, "Arial Bold", 10)
        b.setBackground(Chart.Transparent)
        b.setAlignment(Chart.Right)
        b.setLineStyleKey()

        ' Set the x And y axis stems to transparent And the label font to 10pt Arial
        c.xAxis().setColors(Chart.Transparent)
        c.yAxis().setColors(Chart.Transparent)
        c.xAxis().setLabelStyle("Arial", 10)
        c.yAxis().setLabelStyle("Arial", 10, &H336699)

        ' Configure the y-axis label to be inside the plot area And above the horizontal grid lines
        c.yAxis().setLabelGap(-1)
        c.yAxis().setMargin(20)
        c.yAxis().setLabelAlignment(1)

        ' Configure the x-axis labels to be to the left of the vertical grid lines
        c.xAxis().setLabelAlignment(1)

        '================================================================================
        ' Add data to chart
        '================================================================================

        ' Add line layers using the DataAccelerator. Each layer only supports one accelerated
        ' series, so we add 3 layers for the 3 data series.

        ' Add a line layer for the lines, using a line width of 2 pixels
        Dim layer As LineLayer = c.addLineLayer(fastData, "mA", &HFF0000, "Alpha")
        layer.setLineWidth(2)

        Dim layer2 As LineLayer = c.addLineLayer(fastData, "mB", &HCC00, "Beta")
        layer2.setLineWidth(2)

        Dim layer3 As LineLayer = c.addLineLayer(fastData, "mC", &HFF, "Gamma")
        layer3.setLineWidth(2)

        '================================================================================
        ' Configure axis scale and labelling
        '================================================================================

        ' Set the x-axis as a date/time axis with the scale according to the view port x range.
        viewer.syncLinearAxisWithViewPort("x", c.xAxis())

        ' For the automatic axis labels, set the minimum spacing to 75/40 pixels for the x/y axis.
        c.xAxis().setTickDensity(75)
        c.yAxis().setTickDensity(40)

        ' Set the auto-scale margin to 0.05, And the zero affinity to 0.2
        c.yAxis().setAutoScale(0.05, 0.05, 0.2)

        '================================================================================
        ' Output the chart
        '================================================================================

        ' We need to update the track line too. If the mouse Is moving on the chart (eg. if 
        ' the user drags the mouse on the chart to scroll it), the track line will be updated
        ' in the MouseMovePlotArea event. Otherwise, we need to update the track line here.
        If (Not viewer.IsInMouseMoveEvent) AndAlso viewer.IsMouseOnPlotArea Then
            trackLineLabel(c, viewer.PlotAreaMouseX)
        End If

        viewer.Chart = c

    End Sub

    '
    ' Draw track cursor when mouse is moving over plotarea
    '
    Private Sub winChartViewer1_MouseMovePlotArea(sender As Object, e As MouseEventArgs) _
        Handles winChartViewer1.MouseMovePlotArea

        If Not hasFinishedInitialization Then
            Exit Sub
        End If

        Dim viewer As WinChartViewer = sender
        trackLineLabel(viewer.Chart, viewer.PlotAreaMouseX)
        viewer.updateDisplay()

        ' Hide the track cursor when the mouse leaves the plot area
        viewer.removeDynamicLayer("MouseLeavePlotArea")

    End Sub

    '
    ' Draw track line with data labels
    '
    Private Sub trackLineLabel(c As XYChart, mouseX As Integer)

        ' Clear the current dynamic layer and get the DrawArea object to draw on it.
        Dim d As DrawArea = c.initDynamicLayer()

        ' The plot area object
        Dim plotArea As PlotArea = c.getPlotArea()

        ' Get the data x-value that is nearest to the mouse, and find its pixel coordinate.
        Dim xValue As Double = c.getNearestXValue(mouseX)
        Dim xCoor As Integer = c.getXCoor(xValue)

        ' Draw a vertical track line at the x-position
        d.vline(plotArea.getTopY(), plotArea.getBottomY(), xCoor, d.dashLineColor(&H0, &H101))

        ' Draw a label on the x-axis to show the track line position.
        Dim xlabel As String = "<*font,bgColor=000000*> " & xValue & " <*/font*>"
        Dim t As TTFText = d.text(xlabel, "Arial Bold", 10)

        ' Restrict the x-pixel position of the label to make sure it stays inside the chart image.
        Dim xLabelPos As Integer = Math.Max(0, Math.Min(xCoor - t.getWidth() / 2, c.getWidth() _
            - t.getWidth()))
        t.draw(xLabelPos, plotArea.getBottomY() + 2, &HFFFFFF)

        ' Iterate through all layers to draw the data labels
        For i As Integer = 0 To c.getLayerCount() - 1
            Dim layer As Layer = c.getLayerByZ(i)

            ' The data array index of the x-value
            Dim xIndex As Integer = layer.getXIndexOf(xValue)

            ' Iterate through all the data sets in the layer
            For j As Integer = 0 To layer.getDataSetCount() - 1
                Dim dataSet As ChartDirector.DataSet = layer.getDataSetByZ(j)

                ' Get the color and position of the data label
                Dim color As Integer = dataSet.getDataColor()
                Dim yCoor As Integer = c.getYCoor(dataSet.getPosition(xIndex), dataSet.getUseYAxis())

                ' Draw a track dot with a label next to it for visible data points in the plot area
                If (yCoor >= plotArea.getTopY()) And (yCoor <= plotArea.getBottomY()) And (color <>
                    Chart.Transparent) And (Not String.IsNullOrEmpty(dataSet.getDataName())) Then

                    d.circle(xCoor, yCoor, 4, 4, color, color)

                    Dim label As String = "<*font,bgColor=" & Hex(color) & "*> " & c.formatValue(
                        dataSet.getValue(xIndex), "{value|P4}") & " <*/font*>"
                    t = d.text(label, "Arial Bold", 8)

                    ' Draw the label on the right side of the dot if the mouse is on the left side the chart,
                    ' and vice versa. This ensures the label will not go outside the chart image.
                    If xCoor <= (plotArea.getLeftX() + plotArea.getRightX()) / 2 Then
                        t.draw(xCoor + 5, yCoor, &HFFFFFF, Chart.Left)
                    Else
                        t.draw(xCoor - 5, yCoor, &HFFFFFF, Chart.Right)
                    End If
                End If
            Next
        Next

    End Sub

    Private Sub chartUpdateTimer_Tick(sender As Object, e As EventArgs) _
        Handles chartUpdateTimer.Tick

        ' Append real time data to the data arrays
        If getRealTimeData() Then
            ' Notify the DataAccelerator that New data are appended to the arrays, so it
            ' can accelerate them.
            fastData.extendLength(currentIndex)

            ' We need to update the full x range to include the New data
            updateAxisScale(winChartViewer1)
        End If

    End Sub

    Private Function getRealTimeData() As Boolean

        ' In this example, we simulate a data source that can produced 3 x 1000 data points
        ' per second And store the data in a buffer. When the chart Is updated, it will 
        ' retrieve the data in the buffer.

        ' We determine the time elapsed since last chart update And assume there are already 
        ' the requirement amount of data points in the buffer.
        Dim pointCount As Integer = dataSourceEmulator.ElapsedMilliseconds - lastTimeStamp
        pointCount = Math.Min(pointCount, timeStamps.Length - currentIndex)
        lastTimeStamp += pointCount

        ' We append the data to the arrays
        For i As Integer = currentIndex To currentIndex + pointCount - 1
            timeStamps(i) = i
        Next

        If currentIndex = 0 Then
            ' If the data arrays are empty, just generate some random data series.
            realTimeData.fillSeries2(dataSeriesA, currentIndex, pointCount, 2500, -1, 1)
            realTimeData.fillSeries2(dataSeriesB, currentIndex, pointCount, 2500, -1, 1)
            realTimeData.fillSeries2(dataSeriesC, currentIndex, pointCount, 2500, -1, 1)
        Else
            ' If the data arrays are Not empty, when append random data, ensure it starts from
            ' the last data point to make a continuous series.

            currentIndex -= 1
            pointCount += 1

            realTimeData.fillSeries2(dataSeriesA, currentIndex, pointCount, dataSeriesA(currentIndex), -1, 1)
            realTimeData.fillSeries2(dataSeriesB, currentIndex, pointCount, dataSeriesB(currentIndex), -1, 1)
            realTimeData.fillSeries2(dataSeriesC, currentIndex, pointCount, dataSeriesC(currentIndex), -1, 1)
        End If

        currentIndex += pointCount

        ' Displaying the point count in the pushbutton control to provide some feedback
        plotChartPB.Text = "Point Count = " & currentIndex & " x 3"

        ' Return true if New data are available
        Return pointCount > 0

    End Function

    '
    ' As we added more data, we may need to update the full range. 
    '
    Private Sub updateAxisScale(viewer As WinChartViewer)

        Dim startTime As Double = timeStamps(0)
        Dim endTime As Double = timeStamps(currentIndex - 1)

        ' X-axis range must be at least equal the minXRange.
        Dim duration As Double = endTime - startTime
        If duration < minXRange Then
            endTime = startTime + minXRange
        End If

        ' Update the full range to reflect the actual duration of the data. In this case, 
        ' if the view port Is viewing the latest data, we will scroll the view port as New
        ' data are added. If the view port Is viewing historical data, we would keep the 
        ' axis scale unchanged to keep the chart stable.
        Dim updateType As Integer = IIf(viewer.ViewPortRight < 0.999, Chart.KeepVisibleRange,
            Chart.ScrollWithMax)
        Dim axisScaleHasChanged As Boolean = viewer.updateFullRangeH("x", startTime, endTime,
            updateType)

        ' Set the zoom in limit as a ratio to the full range
        viewer.ZoomInWidthLimit = zoomInLimit / (viewer.getValueAtViewPort("x", 1) -
            viewer.getValueAtViewPort("x", 0))

        ' Trigger the viewPortChanged event to update the display if the axis scale has 
        ' changed Or if New data are added to the existing axis scale.
        If axisScaleHasChanged Or duration < minXRange Then
            viewer.updateViewPort(True, False)
        End If

    End Sub

End Class